"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var immer_1 = require("immer");
var PureStore = /** @class */ (function () {
    function PureStore(root, getter, rootState) {
        var _this = this;
        this.callbacks = [];
        this.getter = null;
        this.getState = function () { return _this.getter(_this.root.rootState); };
        this.update = function (updater) {
            var updaterFn = updater;
            if (!(updater instanceof Function))
                updaterFn = function (e) { return Object.assign(e, updater); };
            _this.root.rootState = immer_1.default(_this.root.rootState, function (s) {
                updaterFn(_this.getter(s));
            });
            _this.callbacks.forEach(function (callback) { return callback(); });
        };
        this.storeFor = function (getter) { return new PureStore(_this, getter); };
        this.updaterFor = function (getter) { return _this.storeFor(getter).update; };
        this.subscribe = function (callback) {
            if (_this.root != _this)
                throw "Only the top level store can be subscribed to.";
            _this.callbacks.push(callback);
            return function () { return _this.callbacks.splice(_this.callbacks.indexOf(callback), 1); };
        };
        this.root = root || this;
        if (!root)
            this.rootState = rootState;
        this.getter = getter || (function (s) { return s; });
    }
    Object.defineProperty(PureStore.prototype, "state", {
        get: function () { return this.getState(); },
        enumerable: true,
        configurable: true
    });
    return PureStore;
}());
exports.PureStore = PureStore;
exports.default = (function (state) { return (new PureStore(null, function (s) { return s; }, state)); });
